@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,16 +19,19 @@
 #ifndef _AO_PINS_H_
 #define _AO_PINS_H_
 
-#include <ao_flash_lpc_pins.h>
+#include <ao_flash_stm_pins.h>
 
-#define AO_BOOT_PIN		1
-#define AO_BOOT_APPLICATION_GPIO	0
-#define AO_BOOT_APPLICATION_PIN		19
+/* pin 5 (PB1) on debug header to gnd for boot mode */
+
+#define AO_BOOT_PIN			1
+#define AO_BOOT_APPLICATION_GPIO	stm_gpiob
+#define AO_BOOT_APPLICATION_PIN		1
 #define AO_BOOT_APPLICATION_VALUE	1
 #define AO_BOOT_APPLICATION_MODE	AO_EXTI_MODE_PULL_UP
 
-#define HAS_USB_PULLUP	1
-#define AO_USB_PULLUP_PORT	0
-#define AO_USB_PULLUP_PIN	20
+/* USB */
+#define HAS_USB			1
+#define AO_USB_DIRECTIO		0
+#define AO_PA11_PA12_RMP	1
 
 #endif /* _AO_PINS_H_ */
