@@ -45,12 +45,15 @@ extern __pdata enum ao_flight_state ao_log_state;
 #define AO_LOG_FORMAT_TELEMETRY		3	/* 32 byte ao_telemetry records */
 #define AO_LOG_FORMAT_TELESCIENCE	4	/* 32 byte typed telescience records */
 #define AO_LOG_FORMAT_TELEMEGA_OLD	5	/* 32 byte typed telemega records */
-#define AO_LOG_FORMAT_EASYMINI		6	/* 16-byte MS5607 baro only, 3.0V supply */
+#define AO_LOG_FORMAT_EASYMINI1		6	/* 16-byte MS5607 baro only, 3.0V supply */
 #define AO_LOG_FORMAT_TELEMETRUM	7	/* 16-byte typed telemetrum records */
-#define AO_LOG_FORMAT_TELEMINI		8	/* 16-byte MS5607 baro only, 3.3V supply */
+#define AO_LOG_FORMAT_TELEMINI2		8	/* 16-byte MS5607 baro only, 3.3V supply, cc1111 SoC */
 #define AO_LOG_FORMAT_TELEGPS		9	/* 32 byte telegps records */
 #define AO_LOG_FORMAT_TELEMEGA		10	/* 32 byte typed telemega records with 32 bit gyro cal */
 #define AO_LOG_FORMAT_DETHERM		11	/* 16-byte MS5607 baro only, no ADC */
+#define AO_LOG_FORMAT_TELEMINI3		12	/* 16-byte MS5607 baro only, 3.3V supply, stm32f042 SoC */
+#define AO_LOG_FORMAT_TELEFIRETWO	13	/* 32-byte test stand data */
+#define AO_LOG_FORMAT_EASYMINI2		14	/* 16-byte MS5607 baro only, 3.3V supply, stm32f042 SoC */
 #define AO_LOG_FORMAT_NONE		127	/* No log at all */
 
 extern __code uint8_t ao_log_format;
@@ -250,8 +253,8 @@ struct ao_log_mega {
 			int16_t		gyro_y;		/* 20 */
 			int16_t		gyro_z;		/* 22 */
 			int16_t		mag_x;		/* 24 */
-			int16_t		mag_y;		/* 26 */
-			int16_t		mag_z;		/* 28 */
+			int16_t		mag_z;		/* 26 */
+			int16_t		mag_y;		/* 28 */
 			int16_t		accel;		/* 30 */
 		} sensor;	/* 32 */
 		/* AO_LOG_TEMP_VOLT */
@@ -299,6 +302,30 @@ struct ao_log_mega {
 						 ((l)->u.gps.altitude_high = (a) >> 16), \
 						 (l)->u.gps.altitude_low = (a))
 
+struct ao_log_firetwo {
+	char			type;			/* 0 */
+	uint8_t			csum;			/* 1 */
+	uint16_t		tick;			/* 2 */
+	union {						/* 4 */
+		/* AO_LOG_FLIGHT */
+		struct {
+			uint16_t	flight;		/* 4 */
+		} flight;	/* 6 */
+		/* AO_LOG_STATE */
+		struct {
+			uint16_t	state;		/* 4 */
+			uint16_t	reason;		/* 6 */
+		} state;	/* 8 */
+		/* AO_LOG_SENSOR */
+		struct {
+			uint16_t	pressure;	/* 4 */
+			uint16_t	thrust;		/* 6 */
+			uint16_t	thermistor[4];	/* 8 */
+		} sensor;	/* 24 */
+		uint8_t		align[28];		/* 4 */
+	} u;	/* 32 */
+};
+
 struct ao_log_metrum {
 	char			type;			/* 0 */
 	uint8_t			csum;			/* 1 */
