@@ -23,8 +23,8 @@ import java.awt.*;
 import java.awt.event.*;
 import javax.swing.*;
 import javax.swing.event.*;
-import org.altusmetrum.altoslib_11.*;
-import org.altusmetrum.altosuilib_11.*;
+import org.altusmetrum.altoslib_12.*;
+import org.altusmetrum.altosuilib_12.*;
 
 public class TeleGPSConfigUI
 	extends AltosUIDialog
@@ -131,7 +131,7 @@ public class TeleGPSConfigUI
 	}
 
 	public double pyro_firing_time() {
-		return -1;
+		return AltosLib.MISSING;
 	}
 
 	boolean is_telemetrum() {
@@ -140,46 +140,46 @@ public class TeleGPSConfigUI
 	}
 
 	void set_radio_enable_tool_tip() {
-		if (radio_enable_value.isEnabled())
+		if (radio_enable_value.isVisible())
 			radio_enable_value.setToolTipText("Enable/Disable telemetry and RDF transmissions");
 		else
 			radio_enable_value.setToolTipText("Firmware version does not support disabling radio");
 	}
 
 	void set_rate_tool_tip() {
-		if (rate_value.isEnabled())
+		if (rate_value.isVisible())
 			rate_value.setToolTipText("Select telemetry baud rate");
 		else
 			rate_value.setToolTipText("Firmware version does not support variable telemetry rates");
 	}
 
 	void set_aprs_interval_tool_tip() {
-		if (aprs_interval_value.isEnabled())
+		if (aprs_interval_value.isVisible())
 			aprs_interval_value.setToolTipText("Enable APRS and set the interval between APRS reports");
 		else
 			aprs_interval_value.setToolTipText("Hardware doesn't support APRS");
 	}
 
 	void set_aprs_ssid_tool_tip() {
-		if (aprs_ssid_value.isEnabled())
+		if (aprs_ssid_value.isVisible())
 			aprs_ssid_value.setToolTipText("Set the APRS SSID (secondary station identifier)");
-		else if (aprs_ssid_value.isEnabled())
+		else if (aprs_ssid_value.isVisible())
 			aprs_ssid_value.setToolTipText("Software version doesn't support setting the APRS SSID");
 		else
 			aprs_ssid_value.setToolTipText("Hardware doesn't support APRS");
 	}
 
 	void set_aprs_format_tool_tip() {
-		if (aprs_format_value.isEnabled())
+		if (aprs_format_value.isVisible())
 			aprs_format_value.setToolTipText("Set the APRS format (compressed/uncompressed)");
-		else if (aprs_format_value.isEnabled())
+		else if (aprs_format_value.isVisible())
 			aprs_format_value.setToolTipText("Software version doesn't support setting the APRS format");
 		else
 			aprs_format_value.setToolTipText("Hardware doesn't support APRS");
 	}
 
 	void set_flight_log_max_tool_tip() {
-		if (flight_log_max_value.isEnabled())
+		if (flight_log_max_value.isVisible())
 			flight_log_max_value.setToolTipText("Size reserved for each flight log (in kB)");
 		else
 			flight_log_max_value.setToolTipText("Cannot set max value with flight logs in memory");
@@ -675,7 +675,7 @@ public class TeleGPSConfigUI
 	public void units_changed(boolean imperial_units) {
 		boolean	was_dirty = dirty;
 
-		if (tracker_motion_value.isEnabled()) {
+		if (tracker_motion_value.isVisible()) {
 			String motion = tracker_motion_value.getSelectedItem().toString();
 			tracker_motion_label.setText(get_tracker_motion_label());
 			set_tracker_motion_values();
@@ -712,21 +712,24 @@ public class TeleGPSConfigUI
 	}
 
 	public int main_deploy() {
-		return -1;
+		return AltosLib.MISSING;
 	}
 
 	public void set_apogee_delay(int new_apogee_delay) { }
 
 	public int apogee_delay() {
-		return -1;
+		return AltosLib.MISSING;
 	}
 
 	public void set_apogee_lockout(int new_apogee_lockout) { }
 
-	public int apogee_lockout() { return -1; }
+	public int apogee_lockout() { return AltosLib.MISSING; }
 
 	public void set_radio_frequency(double new_radio_frequency) {
-		radio_frequency_value.set_frequency(new_radio_frequency);
+		if (new_radio_frequency != AltosLib.MISSING)
+			radio_frequency_value.set_frequency(new_radio_frequency);
+		radio_frequency_label.setVisible(new_radio_frequency != AltosLib.MISSING);
+		radio_frequency_value.setVisible(new_radio_frequency != AltosLib.MISSING);
 	}
 
 	public double radio_frequency() {
@@ -734,11 +737,49 @@ public class TeleGPSConfigUI
 	}
 
 	public void set_radio_calibration(int new_radio_calibration) {
-		radio_calibration_value.setVisible(new_radio_calibration >= 0);
-		if (new_radio_calibration < 0)
-			radio_calibration_value.setText("Disabled");
-		else
+		if (new_radio_calibration != AltosLib.MISSING)
 			radio_calibration_value.setText(String.format("%d", new_radio_calibration));
+		radio_calibration_value.setVisible(new_radio_calibration == AltosLib.MISSING);
+		radio_calibration_label.setVisible(new_radio_calibration == AltosLib.MISSING);
+	}
+
+	public void set_radio_enable(int new_radio_enable) {
+		if (new_radio_enable != AltosLib.MISSING)
+			radio_enable_value.setSelected(new_radio_enable != 0);
+		radio_enable_label.setVisible(new_radio_enable != AltosLib.MISSING);
+		radio_enable_value.setVisible(new_radio_enable != AltosLib.MISSING);
+		set_radio_enable_tool_tip();
+	}
+
+	public int radio_enable() {
+		if (radio_enable_value.isVisible())
+			return radio_enable_value.isSelected() ? 1 : 0;
+		else
+			return AltosLib.MISSING;
+	}
+
+	public void set_telemetry_rate(int new_rate) {
+		if (new_rate != AltosLib.MISSING)
+			rate_value.set_rate(new_rate);
+		rate_label.setVisible(new_rate != AltosLib.MISSING);
+		rate_value.setVisible(new_rate != AltosLib.MISSING);
+	}
+
+	public int telemetry_rate() {
+		return rate_value.rate();
+	}
+
+	public void set_callsign(String new_callsign) {
+		if (new_callsign != null)
+			callsign_value.setText(new_callsign);
+		callsign_value.setVisible(new_callsign != null);
+		callsign_label.setVisible(new_callsign != null);
+	}
+
+	public String callsign() {
+		if (callsign_value.isVisible())
+			return callsign_value.getText();
+		return null;
 	}
 
 	private int parse_int(String name, String s, boolean split) throws AltosConfigDataException {
@@ -752,42 +793,6 @@ public class TeleGPSConfigUI
 		}
 	}
 
-	public void set_radio_enable(int new_radio_enable) {
-		if (new_radio_enable >= 0) {
-			radio_enable_value.setSelected(new_radio_enable > 0);
-			radio_enable_value.setEnabled(true);
-		} else {
-			radio_enable_value.setSelected(true);
-			radio_enable_value.setVisible(radio_frequency() > 0);
-			radio_enable_value.setEnabled(false);
-		}
-		set_radio_enable_tool_tip();
-	}
-
-	public int radio_enable() {
-		if (radio_enable_value.isEnabled())
-			return radio_enable_value.isSelected() ? 1 : 0;
-		else
-			return -1;
-	}
-
-	public void set_telemetry_rate(int new_rate) {
-		rate_value.set_rate(new_rate);
-	}
-
-	public int telemetry_rate() {
-		return rate_value.rate();
-	}
-
-	public void set_callsign(String new_callsign) {
-		callsign_value.setVisible(new_callsign != null);
-		callsign_value.setText(new_callsign);
-	}
-
-	public String callsign() {
-		return callsign_value.getText();
-	}
-
 	int	flight_log_max_limit;
 	int	flight_log_max;
 
@@ -828,15 +833,15 @@ public class TeleGPSConfigUI
 	}
 
 	public void set_ignite_mode(int new_ignite_mode) { }
-	public int ignite_mode() { return -1; }
+	public int ignite_mode() { return AltosLib.MISSING; }
 
 
 	public void set_pad_orientation(int new_pad_orientation) { }
-	public int pad_orientation() { return -1; }
+	public int pad_orientation() { return AltosLib.MISSING; }
 
 	public void set_beep(int new_beep) { }
 
-	public int beep() { return -1; }
+	public int beep() { return AltosLib.MISSING; }
 
 	String[] tracker_motion_values() {
 		if (AltosConvert.imperial_units)
@@ -859,87 +864,92 @@ public class TeleGPSConfigUI
 	}
 
 	void set_tracker_tool_tip() {
-		if (tracker_motion_value.isEnabled())
+		if (tracker_motion_value.isVisible())
 			tracker_motion_value.setToolTipText("How far the device must move before logging");
 		else
 			tracker_motion_value.setToolTipText("This device doesn't disable logging when stationary");
-		if (tracker_interval_value.isEnabled())
+		if (tracker_interval_value.isVisible())
 			tracker_interval_value.setToolTipText("How often to report GPS position");
 		else
 			tracker_interval_value.setToolTipText("This device can't configure interval");
 	}
 
 	public void set_tracker_motion(int tracker_motion) {
-		if (tracker_motion < 0) {
-			tracker_motion_value.setEnabled(false);
-		} else {
-			tracker_motion_value.setEnabled(true);
+		if (tracker_motion != AltosLib.MISSING)
 			tracker_motion_value.setSelectedItem(AltosConvert.height.say(tracker_motion));
-		}
+		tracker_motion_label.setVisible(tracker_motion != AltosLib.MISSING);
+		tracker_motion_value.setVisible(tracker_motion != AltosLib.MISSING);
 	}
 
 	public int tracker_motion() throws AltosConfigDataException {
-		String str = tracker_motion_value.getSelectedItem().toString();
-		try {
-			return (int) (AltosConvert.height.parse_locale(str) + 0.5);
-		} catch (ParseException pe) {
-			throw new AltosConfigDataException("invalid tracker motion %s", str);
+		if (tracker_motion_value.isVisible()) {
+			String str = tracker_motion_value.getSelectedItem().toString();
+			try {
+				return (int) (AltosConvert.height.parse_locale(str) + 0.5);
+			} catch (ParseException pe) {
+				throw new AltosConfigDataException("invalid tracker motion %s", str);
+			}
 		}
+		return AltosLib.MISSING;
 	}
 
 	public void set_tracker_interval(int tracker_interval) {
-		if (tracker_interval< 0) {
-			tracker_interval_value.setEnabled(false);
-		} else {
-			tracker_interval_value.setEnabled(true);
+		if (tracker_interval != AltosLib.MISSING)
 			tracker_interval_value.setSelectedItem(String.format("%d", tracker_interval));
-		}
+		tracker_interval_label.setVisible(tracker_interval != AltosLib.MISSING);
+		tracker_interval_value.setVisible(tracker_interval != AltosLib.MISSING);
 	}
 
 	public int tracker_interval() throws AltosConfigDataException {
-		return parse_int ("tracker interval", tracker_interval_value.getSelectedItem().toString(), false);
+		if (tracker_interval_value.isVisible())
+			return parse_int ("tracker interval", tracker_interval_value.getSelectedItem().toString(), false);
+		return AltosLib.MISSING;
 	}
 
 	public void set_aprs_interval(int new_aprs_interval) {
-		String	s;
-
-		if (new_aprs_interval <= 0)
-			s = "Disabled";
-		else
-			s = Integer.toString(new_aprs_interval);
-		aprs_interval_value.setSelectedItem(s);
-		aprs_interval_value.setVisible(new_aprs_interval >= 0);
+		if (new_aprs_interval != AltosLib.MISSING)
+			aprs_interval_value.setSelectedItem(Integer.toString(new_aprs_interval));
+		aprs_interval_value.setVisible(new_aprs_interval != AltosLib.MISSING);
+		aprs_interval_label.setVisible(new_aprs_interval != AltosLib.MISSING);
 		set_aprs_interval_tool_tip();
 	}
 
 	public int aprs_interval() throws AltosConfigDataException {
-		String	s = aprs_interval_value.getSelectedItem().toString();
+		if (aprs_interval_value.isVisible()) {
+			String	s = aprs_interval_value.getSelectedItem().toString();
 
-		if (s.equals("Disabled"))
-			return 0;
-		return parse_int("aprs interval", s, false);
+			return parse_int("aprs interval", s, false);
+		}
+		return AltosLib.MISSING;
 	}
 
 	public void set_aprs_ssid(int new_aprs_ssid) {
-		aprs_ssid_value.setSelectedItem(Math.max(0,new_aprs_ssid));
-		aprs_ssid_value.setVisible(new_aprs_ssid >= 0);
+		if (new_aprs_ssid != AltosLib.MISSING)
+			aprs_ssid_value.setSelectedItem(new_aprs_ssid);
+		aprs_ssid_value.setVisible(new_aprs_ssid != AltosLib.MISSING);
+		aprs_ssid_label.setVisible(new_aprs_ssid != AltosLib.MISSING);
 		set_aprs_ssid_tool_tip();
 	}
 
 	public int aprs_ssid() throws AltosConfigDataException {
-		Integer i = (Integer) aprs_ssid_value.getSelectedItem();
-		return i;
+		if (aprs_ssid_value.isVisible()) {
+			Integer i = (Integer) aprs_ssid_value.getSelectedItem();
+			return i;
+		}
+		return AltosLib.MISSING;
 	}
 
 	public void set_aprs_format(int new_aprs_format) {
-		aprs_format_value.setVisible(new_aprs_format >= 0);
-		aprs_format_label.setVisible(new_aprs_format >= 0);
-
-		aprs_format_value.setSelectedIndex(Math.max(0,new_aprs_format));
+		if (new_aprs_format != AltosLib.MISSING)
+			aprs_format_value.setSelectedIndex(new_aprs_format);
+		aprs_format_value.setVisible(new_aprs_format != AltosLib.MISSING);
+		aprs_format_label.setVisible(new_aprs_format != AltosLib.MISSING);
 		set_aprs_format_tool_tip();
 	}
 
 	public int aprs_format() throws AltosConfigDataException {
-		return aprs_format_value.getSelectedIndex();
+		if (aprs_format_value.isVisible())
+			return aprs_format_value.getSelectedIndex();
+		return AltosLib.MISSING;
 	}
 }
